import { connect } from 'amqp-connection-manager';
import { URL } from 'url';
import { v4 } from 'uuid';
import { EventEmitter } from 'events';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
}

function __awaiter(thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

var AmqpReceiver = /** @class */ (function () {
    function AmqpReceiver(channel) {
        this.channel = channel;
    }
    AmqpReceiver.prototype.consume = function (queueName, consumer) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.channel.addSetup(function (channel) {
                            return channel.consume(queueName, function (message) {
                                if (message) {
                                    consumer({
                                        headers: message.properties.headers,
                                        data: JSON.parse(message.content.toString())
                                    })
                                        .then(function () { return channel.ack(message); })
                                        .catch(function (err) {
                                        console.error('Error processing message', err);
                                        channel.nack(message);
                                    });
                                }
                            });
                        })];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    return AmqpReceiver;
}());

var AmqpSender = /** @class */ (function () {
    function AmqpSender(channel) {
        this.channel = channel;
    }
    AmqpSender.prototype.declareQueue = function (_a) {
        var name = _a.name, rest = __rest(_a, ["name"]);
        return this.channel.addSetup(function (channel) { return channel.assertQueue(name, rest); });
    };
    AmqpSender.prototype.declareTopic = function (_a) {
        var name = _a.name, type = _a.type;
        return this.channel.addSetup(function (channel) { return channel.assertExchange(name, type); });
    };
    AmqpSender.prototype.bind = function (_a) {
        var queue = _a.queue, topic = _a.topic, routingKey = _a.routingKey;
        return this.channel.addSetup(function (channel) {
            return channel.bindQueue(queue, topic, routingKey);
        });
    };
    AmqpSender.prototype.publish = function (topic, routingKey, message) {
        var content = Buffer.from(JSON.stringify(message.data));
        return this.channel.publish(topic, routingKey, content, {
            deliveryMode: 2,
            headers: message.headers
        });
    };
    return AmqpSender;
}());

var Message = /** @class */ (function () {
    function Message(data, headers) {
        if (headers === void 0) { headers = {}; }
        this.data = data;
        this.headers = headers;
    }
    return Message;
}());

var EventBus = /** @class */ (function () {
    function EventBus(sender, topicName) {
        this.sender = sender;
        this.topicName = topicName;
    }
    EventBus.prototype.emit = function (event) {
        return this.sender.publish(this.topicName, event.name, new Message(event));
    };
    return EventBus;
}());

var AmqpConnection = /** @class */ (function () {
    function AmqpConnection(connectionManager) {
        this.connectionManager = connectionManager;
        this.id = v4().replace(/-/g, '');
        this.channel = connectionManager.createChannel({
            json: false
        });
        this._sender = new AmqpSender(this.channel);
        this._receiver = new AmqpReceiver(this.channel);
        this._eventBus = new EventBus(this._sender, 'domainEvents');
    }
    Object.defineProperty(AmqpConnection.prototype, "sender", {
        get: function () {
            return this._sender;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AmqpConnection.prototype, "receiver", {
        get: function () {
            return this._receiver;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AmqpConnection.prototype, "eventBus", {
        get: function () {
            return this._eventBus;
        },
        enumerable: true,
        configurable: true
    });
    AmqpConnection.prototype.onConnect = function (listener) {
        this.connectionManager.on('connect', function (connected) {
            var url = new URL(connected.url);
            listener({ host: url.host });
        });
    };
    AmqpConnection.prototype.onDisconnect = function (listener) {
        this.connectionManager.on('disconnect', function (err) { return listener(err.err); });
    };
    return AmqpConnection;
}());

var AmqpBroker = /** @class */ (function () {
    function AmqpBroker(url, options) {
        this.url = url;
        this.options = options;
    }
    AmqpBroker.prototype.connect = function () {
        var connection = connect([this.url], { connectionOptions: this.options });
        return new AmqpConnection(connection);
    };
    return AmqpBroker;
}());

var HandlerRegistry = /** @class */ (function () {
    function HandlerRegistry() {
        this._commandHandlers = {};
        this._queryHandlers = {};
        this._eventListeners = {};
        this.eventListenersUpdates = new EventEmitter();
    }
    Object.defineProperty(HandlerRegistry.prototype, "commandHandlers", {
        get: function () {
            return this._commandHandlers;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HandlerRegistry.prototype, "queryHandlers", {
        get: function () {
            return this._queryHandlers;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HandlerRegistry.prototype, "eventListeners", {
        get: function () {
            return this._eventListeners;
        },
        enumerable: true,
        configurable: true
    });
    HandlerRegistry.register = function () {
        return new HandlerRegistry();
    };
    HandlerRegistry.prototype.getCommandHandler = function (name) {
        return this.commandHandlers[name];
    };
    HandlerRegistry.prototype.getQueryHandler = function (resource) {
        return this.queryHandlers[resource];
    };
    HandlerRegistry.prototype.getEventListeners = function (name) {
        return this.eventListeners[name] || [];
    };
    HandlerRegistry.prototype.handleCommand = function (name, handler) {
        if (this.commandHandlers[name]) {
            throw new Error("Command " + name + " already registered");
        }
        this.commandHandlers[name] = handler;
    };
    HandlerRegistry.prototype.serveQuery = function (resource, handler) {
        if (this.queryHandlers[resource]) {
            throw new Error("Query " + resource + " already registered");
        }
        this.queryHandlers[resource] = handler;
    };
    HandlerRegistry.prototype.listenEvent = function (name, listener) {
        this.push(this.eventListeners, name, listener);
        this.eventListenersUpdates.emit('eventListenerPushed', name, listener);
    };
    HandlerRegistry.prototype.onEventListenerPushed = function (handle) {
        this.eventListenersUpdates.on('eventListenerPushed', handle);
    };
    HandlerRegistry.prototype.push = function (routes, path, handler) {
        if (!routes[path]) {
            routes[path] = [];
        }
        routes[path].push(handler);
    };
    return HandlerRegistry;
}());

var Command = /** @class */ (function () {
    function Command(commandId, name, data) {
        this.commandId = commandId;
        this.name = name;
        this.data = data;
    }
    return Command;
}());
var Query = /** @class */ (function () {
    function Query(queryId, resource, queryData) {
        this.queryId = queryId;
        this.resource = resource;
        this.queryData = queryData;
    }
    return Query;
}());
var Event = /** @class */ (function () {
    function Event(eventId, name, data) {
        this.eventId = eventId;
        this.name = name;
        this.data = data;
    }
    return Event;
}());

function createCommand(name) {
    return function (id, data) { return new Command(id, name, data); };
}
function createQuery(name) {
    return function (id, data) { return new Query(id, name, data); };
}
function createEvent(name) {
    return function (id, data) { return new Event(id, name, data); };
}

var EventListener = /** @class */ (function () {
    function EventListener(config, connection, registry) {
        this.config = config;
        this.connection = connection;
        this.registry = registry;
    }
    EventListener.prototype.setupResources = function () {
        var queueName = this.config.appName + ".subscribedEvents";
        var topicName = 'domainEvents';
        this.listenForNewListeners(queueName, topicName);
        return Promise.all([
            this.declareEventsQueue(queueName),
            this.declareEventsTopic(topicName),
            this.setupBindings(queueName, topicName)
        ]).then();
    };
    EventListener.prototype.startListening = function () {
        var _this = this;
        return this.connection.receiver.consume(this.config.appName + ".subscribedEvents", function (message) {
            var event = message.data;
            var listeners = _this.registry
                .getEventListeners(event.name)
                .map(function (listener) { return listener(event); });
            return Promise.all(listeners).then();
        });
    };
    EventListener.prototype.listenForNewListeners = function (queueName, topicName) {
        var _this = this;
        this.registry.onEventListenerPushed(function (name) {
            _this.connection.sender
                .bind({
                queue: queueName,
                topic: topicName,
                routingKey: name
            })
                .catch(function (error) { return console.error(error); });
        });
    };
    EventListener.prototype.declareEventsQueue = function (queueName) {
        return this.connection.sender.declareQueue({
            name: queueName
        });
    };
    EventListener.prototype.declareEventsTopic = function (topicName) {
        return this.connection.sender.declareTopic({
            name: topicName,
            type: 'topic'
        });
    };
    EventListener.prototype.setupBindings = function (queueName, topicName) {
        var _this = this;
        var eventListeners = Object.keys(this.registry.eventListeners).map(function (eventName) {
            return _this.connection.sender.bind({
                queue: queueName,
                topic: topicName,
                routingKey: eventName
            });
        });
        return Promise.all(eventListeners).then();
    };
    return EventListener;
}());

var REPLY_ID = 'x-reply_id';
var CORRELATION_ID = 'x-correlation-id';
var COMPLETION_ONLY_SIGNAL = 'x-empty-completion';

var QueryListener = /** @class */ (function () {
    function QueryListener(config, connection, registry) {
        this.config = config;
        this.connection = connection;
        this.registry = registry;
    }
    QueryListener.prototype.setupResources = function () {
        var queueName = this.config.appName + ".queries";
        var topicName = 'directMessages';
        var routingKey = this.config.appName + ".query";
        return Promise.all([
            this.declareQueriesQueue(queueName),
            this.declareDirectMessagesTopic(topicName),
            this.setupBindings(queueName, topicName, routingKey)
        ]).then();
    };
    QueryListener.prototype.startListening = function () {
        var _this = this;
        return this.connection.receiver.consume(this.config.appName + ".queries", function (message) {
            var query = message.data;
            var handler = _this.registry.getQueryHandler(query.resource);
            return handler(query).then(function (response) { return _this.publishResponse(message, response); });
        });
    };
    QueryListener.prototype.publishResponse = function (request, response) {
        var _a;
        var replyId = request.headers[REPLY_ID];
        var correlationId = request.headers[CORRELATION_ID];
        var responseHeaders = (_a = {},
            _a[CORRELATION_ID] = correlationId,
            _a);
        if (!response) {
            responseHeaders[COMPLETION_ONLY_SIGNAL] = '1';
        }
        return this.connection.sender.publish('globalReply', replyId, new Message(response, responseHeaders));
    };
    QueryListener.prototype.declareQueriesQueue = function (queueName) {
        return this.connection.sender.declareQueue({
            name: queueName
        });
    };
    QueryListener.prototype.declareDirectMessagesTopic = function (topicName) {
        return this.connection.sender.declareTopic({
            name: topicName,
            type: 'direct'
        });
    };
    QueryListener.prototype.setupBindings = function (queueName, topicName, routingKey) {
        return this.connection.sender.bind({
            queue: queueName,
            topic: topicName,
            routingKey: routingKey
        });
    };
    return QueryListener;
}());

var ReactiveCommons = /** @class */ (function () {
    function ReactiveCommons(broker, registry, config) {
        this.broker = broker;
        this.registry = registry;
        this.config = config;
    }
    ReactiveCommons.prototype.start = function () {
        var connection = this.broker.connect();
        var eventListener = new EventListener(this.config, connection, this.registry);
        var queryListener = new QueryListener(this.config, connection, this.registry);
        return Promise.all([
            eventListener.setupResources(),
            queryListener.setupResources(),
            eventListener.startListening(),
            queryListener.startListening()
        ]).then(function () { return connection; });
    };
    return ReactiveCommons;
}());

export { AmqpBroker, AmqpConnection, AmqpReceiver, AmqpSender, Command, Event, HandlerRegistry, Message, Query, ReactiveCommons, createCommand, createEvent, createQuery };
//# sourceMappingURL=index.es5.js.map
