"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var url_1 = require("url");
var uuid_1 = require("uuid");
var amqp_receiver_1 = require("./amqp-receiver");
var amqp_sender_1 = require("./amqp-sender");
var event_bus_1 = require("../../domain/api/emitters/event-bus");
var AmqpConnection = /** @class */ (function () {
    function AmqpConnection(connectionManager) {
        this.connectionManager = connectionManager;
        this.id = uuid_1.v4().replace(/-/g, '');
        this.channel = connectionManager.createChannel({
            json: false
        });
        this._sender = new amqp_sender_1.AmqpSender(this.channel);
        this._receiver = new amqp_receiver_1.AmqpReceiver(this.channel);
        this._eventBus = new event_bus_1.EventBus(this._sender, 'domainEvents');
    }
    Object.defineProperty(AmqpConnection.prototype, "sender", {
        get: function () {
            return this._sender;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AmqpConnection.prototype, "receiver", {
        get: function () {
            return this._receiver;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AmqpConnection.prototype, "eventBus", {
        get: function () {
            return this._eventBus;
        },
        enumerable: true,
        configurable: true
    });
    AmqpConnection.prototype.onConnect = function (listener) {
        this.connectionManager.on('connect', function (connected) {
            var url = new url_1.URL(connected.url);
            listener({ host: url.host });
        });
    };
    AmqpConnection.prototype.onDisconnect = function (listener) {
        this.connectionManager.on('disconnect', function (err) { return listener(err.err); });
    };
    return AmqpConnection;
}());
exports.AmqpConnection = AmqpConnection;
//# sourceMappingURL=amqp-connection.js.map