"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var events_1 = require("events");
var HandlerRegistry = /** @class */ (function () {
    function HandlerRegistry() {
        this._commandHandlers = {};
        this._queryHandlers = {};
        this._eventListeners = {};
        this.eventListenersUpdates = new events_1.EventEmitter();
    }
    Object.defineProperty(HandlerRegistry.prototype, "commandHandlers", {
        get: function () {
            return this._commandHandlers;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HandlerRegistry.prototype, "queryHandlers", {
        get: function () {
            return this._queryHandlers;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HandlerRegistry.prototype, "eventListeners", {
        get: function () {
            return this._eventListeners;
        },
        enumerable: true,
        configurable: true
    });
    HandlerRegistry.register = function () {
        return new HandlerRegistry();
    };
    HandlerRegistry.prototype.getCommandHandler = function (name) {
        return this.commandHandlers[name];
    };
    HandlerRegistry.prototype.getQueryHandler = function (resource) {
        return this.queryHandlers[resource];
    };
    HandlerRegistry.prototype.getEventListeners = function (name) {
        return this.eventListeners[name] || [];
    };
    HandlerRegistry.prototype.handleCommand = function (name, handler) {
        if (this.commandHandlers[name]) {
            throw new Error("Command " + name + " already registered");
        }
        this.commandHandlers[name] = handler;
    };
    HandlerRegistry.prototype.serveQuery = function (resource, handler) {
        if (this.queryHandlers[resource]) {
            throw new Error("Query " + resource + " already registered");
        }
        this.queryHandlers[resource] = handler;
    };
    HandlerRegistry.prototype.listenEvent = function (name, listener) {
        this.push(this.eventListeners, name, listener);
        this.eventListenersUpdates.emit('eventListenerPushed', name, listener);
    };
    HandlerRegistry.prototype.onEventListenerPushed = function (handle) {
        this.eventListenersUpdates.on('eventListenerPushed', handle);
    };
    HandlerRegistry.prototype.push = function (routes, path, handler) {
        if (!routes[path]) {
            routes[path] = [];
        }
        routes[path].push(handler);
    };
    return HandlerRegistry;
}());
exports.HandlerRegistry = HandlerRegistry;
//# sourceMappingURL=handler-registry.js.map