"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var EventListener = /** @class */ (function () {
    function EventListener(config, connection, registry) {
        this.config = config;
        this.connection = connection;
        this.registry = registry;
    }
    EventListener.prototype.setupResources = function () {
        var queueName = this.config.appName + ".subscribedEvents";
        var topicName = 'domainEvents';
        this.listenForNewListeners(queueName, topicName);
        return Promise.all([
            this.declareEventsQueue(queueName),
            this.declareEventsTopic(topicName),
            this.setupBindings(queueName, topicName)
        ]).then();
    };
    EventListener.prototype.startListening = function () {
        var _this = this;
        return this.connection.receiver.consume(this.config.appName + ".subscribedEvents", function (message) {
            var event = message.data;
            var listeners = _this.registry
                .getEventListeners(event.name)
                .map(function (listener) { return listener(event); });
            return Promise.all(listeners).then();
        });
    };
    EventListener.prototype.listenForNewListeners = function (queueName, topicName) {
        var _this = this;
        this.registry.onEventListenerPushed(function (name) {
            _this.connection.sender
                .bind({
                queue: queueName,
                topic: topicName,
                routingKey: name
            })
                .catch(function (error) { return console.error(error); });
        });
    };
    EventListener.prototype.declareEventsQueue = function (queueName) {
        return this.connection.sender.declareQueue({
            name: queueName
        });
    };
    EventListener.prototype.declareEventsTopic = function (topicName) {
        return this.connection.sender.declareTopic({
            name: topicName,
            type: 'topic'
        });
    };
    EventListener.prototype.setupBindings = function (queueName, topicName) {
        var _this = this;
        var eventListeners = Object.keys(this.registry.eventListeners).map(function (eventName) {
            return _this.connection.sender.bind({
                queue: queueName,
                topic: topicName,
                routingKey: eventName
            });
        });
        return Promise.all(eventListeners).then();
    };
    return EventListener;
}());
exports.EventListener = EventListener;
//# sourceMappingURL=event-listener.js.map