"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var broker_model_1 = require("../../model/broker.model");
var headers_model_1 = require("../../model/headers.model");
var QueryListener = /** @class */ (function () {
    function QueryListener(config, connection, registry) {
        this.config = config;
        this.connection = connection;
        this.registry = registry;
    }
    QueryListener.prototype.setupResources = function () {
        var queueName = this.config.appName + ".queries";
        var topicName = 'directMessages';
        var routingKey = this.config.appName + ".query";
        return Promise.all([
            this.declareQueriesQueue(queueName),
            this.declareDirectMessagesTopic(topicName),
            this.setupBindings(queueName, topicName, routingKey)
        ]).then();
    };
    QueryListener.prototype.startListening = function () {
        var _this = this;
        return this.connection.receiver.consume(this.config.appName + ".queries", function (message) {
            var query = message.data;
            var handler = _this.registry.getQueryHandler(query.resource);
            return handler(query).then(function (response) { return _this.publishResponse(message, response); });
        });
    };
    QueryListener.prototype.publishResponse = function (request, response) {
        var _a;
        var replyId = request.headers[headers_model_1.REPLY_ID];
        var correlationId = request.headers[headers_model_1.CORRELATION_ID];
        var responseHeaders = (_a = {},
            _a[headers_model_1.CORRELATION_ID] = correlationId,
            _a);
        if (!response) {
            responseHeaders[headers_model_1.COMPLETION_ONLY_SIGNAL] = '1';
        }
        return this.connection.sender.publish('globalReply', replyId, new broker_model_1.Message(response, responseHeaders));
    };
    QueryListener.prototype.declareQueriesQueue = function (queueName) {
        return this.connection.sender.declareQueue({
            name: queueName
        });
    };
    QueryListener.prototype.declareDirectMessagesTopic = function (topicName) {
        return this.connection.sender.declareTopic({
            name: topicName,
            type: 'direct'
        });
    };
    QueryListener.prototype.setupBindings = function (queueName, topicName, routingKey) {
        return this.connection.sender.bind({
            queue: queueName,
            topic: topicName,
            routingKey: routingKey
        });
    };
    return QueryListener;
}());
exports.QueryListener = QueryListener;
//# sourceMappingURL=query-listener.js.map