"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ReplyListener = /** @class */ (function () {
    function ReplyListener(config, connection, registry) {
        this.config = config;
        this.connection = connection;
        this.registry = registry;
        this.queueName = this.config.appName + "." + this.connection.id + ".replies";
        this.topicName = 'globalReply';
        this.routingKey = this.config.appName;
    }
    ReplyListener.prototype.setupResources = function () {
        return Promise.all([
            this.declareRepliesQueue(this.queueName),
            this.declareGlobalReplyTopic(this.topicName),
            this.setupBindings(this.queueName, this.topicName, this.routingKey)
        ]).then();
    };
    ReplyListener.prototype.startListening = function () {
        return this.connection.receiver.consume(this.queueName, function (message) {
            return Promise.resolve();
        });
    };
    ReplyListener.prototype.declareRepliesQueue = function (queueName) {
        return this.connection.sender.declareQueue({
            name: queueName,
            exclusive: true
        });
    };
    ReplyListener.prototype.declareGlobalReplyTopic = function (topicName) {
        return this.connection.sender.declareTopic({
            name: topicName,
            type: 'topic'
        });
    };
    ReplyListener.prototype.setupBindings = function (queueName, topicName, routingKey) {
        return this.connection.sender.bind({
            queue: queueName,
            topic: topicName,
            routingKey: routingKey
        });
    };
    return ReplyListener;
}());
exports.ReplyListener = ReplyListener;
//# sourceMappingURL=reply-listener.js.map